package com.softslate.commerce.businessobjects.core;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;
import java.util.regex.Pattern;

/**
 * Static methods supporting "YYYY-MM-DD" dates.
 * 
 * @author jtobey, jmcwill
 * 
 */
public class DateUtils {

	private static final Pattern DATE_PATTERN = Pattern
			.compile("\\A\\d\\d\\d\\d-\\d\\d-\\d\\d\\z");

	// The useful property of UTC here is its lack of daylight savings shifts.
	private static final Calendar EPOCH = Calendar.getInstance(TimeZone
			.getTimeZone("UTC"), Locale.US);
	static {
		EPOCH.setTimeInMillis(0L);
	}

	public static void dateSet(Calendar cal, String date) {
		if (!DATE_PATTERN.matcher(date).matches())
			throw new IllegalArgumentException(
					"malformed date, must be YYYY-MM-DD: \"" + date + "\"");
		cal.set(Integer.parseInt(date.substring(0, 4)), Integer.parseInt(date
				.substring(5, 7)) - 1, Integer.parseInt(date.substring(8, 10)));
	}

	public static String dateGet(Calendar cal) {
		return String.format("%04d-%02d-%02d", cal.get(Calendar.YEAR), cal
				.get(Calendar.MONTH) + 1, cal.get(Calendar.DAY_OF_MONTH));
	}

	public static int dateSinceEpoch(String date) {
		Calendar cal = (Calendar) EPOCH.clone();
		dateSet(cal, date);
		return (int) (cal.getTimeInMillis() / (24 * 60 * 60 * 1000));
	}

	public static int dateSubtract(String endDate, String startDate) {
		return dateSinceEpoch(endDate) - dateSinceEpoch(startDate);
	}

	public static String dateAdd(String date, int delta) {
		if (delta == 0)
			return date;
		Calendar cal = (Calendar) EPOCH.clone();
		cal.add(Calendar.DAY_OF_MONTH, dateSinceEpoch(date) + delta);
		return dateGet(cal);
	}

	public static boolean isDate(String date) {
		try {
			return (date != null && date.equals(dateAdd(dateGet(EPOCH),
					dateSinceEpoch(date))));
		} catch (IllegalArgumentException e) {
			return false;
		}
	}

	public static String truncate(String time) {
		if (time == null || time.length() < 10)
			return null;
		return time.substring(0, 10);
	}

	public static Date dateFromString(String date) {
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd", Locale
				.getDefault());
		try {
			return formatter.parse(date);
		} catch (ParseException e) {
			throw new IllegalArgumentException(
					"malformed date, must be YYYY-MM-DD: \"" + date + "\"");
		}
	}

	public static String stringFromDate(Date date) {
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd", Locale
				.getDefault());
		return formatter.format(date);
	}

	public static Calendar calendarFromString(String date) {
		Calendar dateCal = Calendar.getInstance();
		dateCal.setTimeInMillis(dateFromString(date).getTime());
		return dateCal;
	}

	public static String stringFromCalendar(Calendar date) {
		return dateGet(date);
	}

	public static final String[] DAYS = new String[] { "SATURDAY", "SUNDAY",
			"MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY" };


	public static String formatDayMonthDate(String date) {
		Calendar cal = DateUtils.calendarFromString(date);
		return new SimpleDateFormat("EEEE, MMMM dd").format(cal.getTime());
	}

	public static String todaysDateString() {
		Calendar dateCal = Calendar.getInstance();
		return dateGet(dateCal);
	}

	public static Date dateTimeFromString(String datetime) {
		SimpleDateFormat formatter = new SimpleDateFormat(
				"yyyy-MM-dd HH:mm:ss", Locale.getDefault());
		try {
			return formatter.parse(datetime);
		} catch (ParseException e) {
			throw new IllegalArgumentException(
					"malformed date, must be yyyy-MM-dd HH:mm:ss: \""
							+ datetime + "\"");
		}
	}

	public static Calendar calendarFromDateTimeString(String datetime) {
		Calendar dateCal = Calendar.getInstance();
		dateCal.setTimeInMillis(dateTimeFromString(datetime).getTime());
		return dateCal;
	}

	public static String dateTimeAdd(String datetime, int hours, int minutes,
			int seconds) {
		Calendar cal = calendarFromDateTimeString(datetime);
		cal.add(Calendar.HOUR, hours);
		cal.add(Calendar.MINUTE, minutes);
		cal.add(Calendar.SECOND, seconds);
		return dateTimeGet(cal);
	}

	public static String dateTimeGet(Calendar cal) {
		SimpleDateFormat formatter = new SimpleDateFormat(
				"yyyy-MM-dd HH:mm:ss", Locale.getDefault());
		return formatter.format(cal.getTime());
	}

	public static String timeGet(Calendar cal) {
		SimpleDateFormat formatter = new SimpleDateFormat("HH:mm:ss", Locale.getDefault());
		return formatter.format(cal.getTime());
	}

	public static String dateTimeGet(Calendar cal, boolean hour, boolean min,
			boolean sec) {
		ArrayList<String> formatStrings = new ArrayList<String>();

		if (hour)
			formatStrings.add("HH");
		if (min)
			formatStrings.add("mm");
		if (sec)
			formatStrings.add("ss");

		StringBuilder formatString = new StringBuilder("");
		if (formatStrings.size() > 0) {
			formatString.append(" ");
			for (int i = 0; i < formatStrings.size(); i++) {
				if (i != 0)
					formatString.append(":");
				formatString.append(formatStrings.get(i));
			}
		}
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd"
				+ formatString.toString(), Locale.getDefault());
		return formatter.format(cal.getTime());
	}

	/**
	 * Appends all or part of " 00:00:00" to the argument to return a 16-character string.
	 * {@code datetime} may have days, hours, minutes, or seconds resolution.
	 * @param datetime - YYYY-MM-DD[ hh[:mm[:ss]]]
	 * @return YYYY-MM-DD hh:mm:ss
	 */
	public static String appendSeconds(String datetime) {
		if (datetime != null) {
			switch (datetime.length()) {
			case 10: return datetime + " 00:00:00";
			case 13: return datetime + ":00:00";
			case 16: return datetime + ":00";
			}
		}
		return datetime;
	}

	public static String appendMinutes(String datetime) {
		return datetime == null ? null : appendSeconds(datetime).substring(0, 16);
	}


	/**
	 * Return the current server time as {@code YYYY-MM-DD hh:mm:ss}.
	 * @return current date and time
	 */
	public static String now() {
		return dateTimeGet(Calendar.getInstance());
	}

	/**
	 * If the arguments look like a date and time (YYYY-MM-DD and hh:mm), return the full time string
	 * formed by concatenating them with a space between.  Otherwise, just return {@code date}.
	 * @param date - YYYY-MM-DD
	 * @param time - hh:mm
	 * @return YYYY-MM-DD hh:mm
	 */
	public static String appendTime(String date, String time) {
		if (date != null && date.length() == 10 && time != null && time.length() >= 5) {
			return date + " " + time;
		}
		return date;
	}

	public static int dayOfWeek(String date) {
		return calendarFromString(date).get(Calendar.DAY_OF_WEEK);
	}


	/**
	 * Add dashes to a date formatted like yyyyMMdd to send to the database.
	 * 
	 * @param dateToFormat
	 * @return YYYY-MM-DD
	 * @throws IllegalArgumentException
	 */
	public static String addDateDashes(String dateToFormat) throws IllegalArgumentException {
		SimpleDateFormat parseDate = new java.text.SimpleDateFormat("yyyyMMdd");
		SimpleDateFormat formatDate = new java.text.SimpleDateFormat("yyyy-MM-dd");
		try {
			Date date = (Date) parseDate.parse(dateToFormat);
			String newDate = formatDate.format(date);
			return newDate;
		} catch (ParseException ex) {
			throw new IllegalArgumentException("malformed date, must be yyyyMMdd " + dateToFormat);
		}
	}

	public static String addDateTimeDashesColons(String dateToFormat) throws IllegalArgumentException {
		SimpleDateFormat parseDate = new java.text.SimpleDateFormat("yyyyMMdd HHmmss");
		SimpleDateFormat formatDate = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		try {
			Date date = (Date) parseDate.parse(dateToFormat);
			String newDate = formatDate.format(date);
			return newDate;
		} catch (ParseException ex) {
			throw new IllegalArgumentException("malformed date time, must be yyyyMMdd HHmmss" + dateToFormat);
		}
	}

}
