package com.softslate.commerce.businessobjects.core;

import java.io.Serializable;
import java.util.Collection;
import java.util.Map;

/**
 * Interface representing application settings stored in the database.
 * <p>
 * While {@link Setting} is used as a direct interface to the
 * <code>sscSetting</code> database table, this interface's methods are what
 * the rest of the application actually uses to retrieve settings. The
 * <code>getValue</code> method looks up a given setting in a <code>Map</code>
 * returned by this interface's <code>getMap</code> method. The keys of this
 * <code>Map</code> correspond to the <code>code</code> field of the
 * <code>sscSetting</code> database table. The values correspond to the value
 * field (<code>integerValue</code>, <code>smallValue</code>,
 * <code>mediumValue</code>, or <code>textValue</code>) that corresponds
 * with the value of the <code>valueType</code> field.
 * <p>
 * The default "settingsImplementer" is
 * {@link com.softslate.commerce.businessobjects.core.SettingsBean}.
 * 
 * @author David Tobey
 */
public interface Settings extends BusinessObject, Serializable {
	/**
	 * Retrieves the map of settings values for this <code>Settings</code>
	 * instance. The keys of this <code>Map</code> correspond to the
	 * <code>code</code> field of the <code>sscSetting</code> database
	 * table. The values correspond to value field (<code>integerValue</code>,
	 * <code>smallValue</code>, <code>mediumValue</code>, or
	 * <code>textValue</code>) that corresponds with the value of the
	 * <code>valueType</code> field.
	 * 
	 * @return The <code>Map</code> of settings keys and values for this
	 *         <code>Settings</code> instance.
	 */
	abstract Map getMap();

	/**
	 * Sets the map of settings values for this <code>Settings</code>
	 * instance.
	 */
	abstract void setMap(Map map);

	/**
	 * Sets the value of a given key in this <code>Settings</code>'
	 * <code>map</code>.
	 * 
	 * @param key
	 *            The key whose value is being set
	 * @param value
	 *            The value.
	 */
	abstract void setValue(String key, String value);

	/**
	 * Gets the value of a given key in this <code>Settings</code>'
	 * <code>map</code>.
	 * 
	 * @param key
	 *            The key whose value is to be retrieved.
	 * @return The value of the setting.
	 */
	abstract String getValue(String key);

	/**
	 * Initializes this <code>Settings</code> instance. Populates the
	 * <code>map</code> property with the results of a query on the
	 * <code>sscSetting</code> database table.
	 * 
	 * @throws Exception
	 */
	abstract void initialize() throws Exception;

	/**
	 * Returns a new Settings object that is a copy of this one, but with a
	 * given element altered. The incoming name and value represent the element
	 * that is to altered and the new value for it. This is
	 * useful in custom code if in an isolated area different settings are
	 * needed to perform some custom action.
	 * 
	 * @param name
	 * @param value
	 * @return A new Settings object with the new setting value applied to it.
	 * @throws Exception
	 */
	abstract Settings copyAndAlter(String name, String value) throws Exception;

	/**
	 * Returns a new Settings object that is a copy of this one, but with
	 * certain values altered, as represented in the incoming Map. 
	 * 
	 * @param settingsToAlter A Map of new settings to apply to the new object.
	 * @return A new Settings object with the new setting value applied to it.
	 * @throws Exception
	 */
	abstract Settings copyAndAlter(Map settingsToAlter) throws Exception;

	abstract Map loadSettingsAndCount(Map parameters) throws Exception;
	
	abstract Map addSetting(Map parameters) throws Exception;
	
	abstract Setting loadSettingFromID(Map paramaters) throws Exception;
	
	abstract Map editSetting(Map parameters) throws Exception;
	
	abstract void deleteSetting(Map parameters) throws Exception;

	abstract Map editDeleteUserSettings(Map parameters) throws Exception;
	
	abstract Settings processSettings(Map parameters) throws Exception;

	abstract Collection loadSettings() throws Exception;

	abstract Map postChangingSettingEvent(Setting setting, Map parameters) throws Exception;
}
