package com.softslate.commerce.businessobjects.order;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods affecting orders and other related
 * objects.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.BasicOrderProcessor}.
 * 
 * @author David Tobey
 */
public interface OrderProcessor extends BusinessProcessor {

	/**
	 * Loads the store's states and countries from the database. The Struts
	 * layer calls this method when the application starts. The data is kept in
	 * the servlet's application scope and displayed to users filling in address
	 * information.
	 * 
	 * @return A {@link StatesAndCountries} object containing
	 *         <code>Collection</code> s of the store's states and countries.
	 * @throws Exception
	 */
	abstract StatesAndCountries loadStatesAndCountries() throws Exception;

	abstract Collection loadDashboardStatistics() throws Exception;

	abstract Collection loadIncompleteOrdersStatistics() throws Exception;

	abstract int deleteIncompleteOrders(Map parameters) throws Exception;

	abstract Map loadOrdersAndCount(Map parameters) throws Exception;

	abstract Map addOrder(Map parameters) throws Exception;

	abstract Order loadOrderFromID(Map parameters) throws Exception;

	abstract Map editOrder(Map parameters) throws Exception;

	abstract Map editDeleteOrders(Map parameters) throws Exception;

	abstract void deleteOrder(Map parameters) throws Exception;

	abstract Order loadOrderDetails(int orderNumber) throws Exception;

	abstract Map loadSavedCartFromToken(Map parameters) throws Exception;

	abstract int loadMaxOrderID() throws Exception;

	/**
	 * Duplicates an <code>Order</code> object. Performs a deep copy of its
	 * dependancies (excluding primary keys). Used for picking up saved carts.
	 * 
	 * @param order
	 *            The <code>Order</code> to be duplicated.
	 * @return The duplicated <code>Order</code> object.
	 * @throws Exception
	 */
	abstract Order duplicateOrder(Order order) throws Exception;

	/**
	 * Reprocesses the items in an <code>Order</code> object by adding them to
	 * the user's current cart. Used for picking up saved carts.
	 * 
	 * @param order
	 *            The <code>Order</code> whose items are to be reprocessed.
	 * @return a <code>Map</code> containing the results of the reprocessing.
	 * @throws Exception
	 */
	abstract Map reprocessOrder(Order order) throws Exception;

	/**
	 * Queries the system to find orders which have payments but have not been
	 * marked complete. This conditions indicates that a payment was submitted
	 * but something went wrong during the process.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing <code>String</code> values
	 *            olderThanDays, olderThanHours which can be parsed to an
	 *            <code>Integer</code>.
	 * @return a <code>Collection</code> containing the orders if any.
	 * @throws Exception
	 */
	abstract Collection loadIncompleteOrdersWithPayments(Map parameters)
			throws Exception;

	abstract Map postChangingOrderEvent(Order item, Map parameters) throws Exception;

	abstract Map postChangedOrderEvent(Order order, Map parameters) throws Exception;

	abstract Order loadOrderFromNumber(Map parameters) throws Exception;

	abstract Order loadOrderFromNumber(int orderNumber) throws Exception;
	
	abstract Collection<Order> loadCompletedOrdersByDateRange(String startDate, String endDate,
			Collection<String> ignoreStatuses) throws Exception;

	abstract Collection<Order> loadAbandonedCartOrders(String startDate, String endDate, String prevOrdersDate)
			throws Exception;

	abstract Collection<Order> loadCompletedOrdersByLastModifiedAndStatuses(String startDate, String endDate,
			Collection<String> currentStatuses) throws Exception;

}
