package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class used for requests to log into a customer account. Used
 * for requests to "/Login.do".
 * <p>
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#processLogin(Map)}
 * to process the login request. If successful, that method will have populated
 * the <code>customer</code> property of the
 * {@link com.softslate.commerce.businessobjects.core.User} instance in the
 * current user's session.
 * <p>
 * If a valid customer account cannot be found in the database, the
 * <code>ActionForward</code> named "failure" will be returned.
 * 
 * @author David Tobey
 * 
 */
public class LoginAction extends BaseAction {
	static Log log = LogFactory.getLog(LoginAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = (BaseForm) form;
		synchronized (baseForm.getRequest().getSession()) {

			// Prepare the autoLoginToken, if called for
			boolean setAutoLoginCookie = false;
			Map parameters = PropertyUtils.describe(baseForm);
			String allowCustomerAutoLogins = (String) baseForm.getSettings()
					.getValue("allowCustomerAutoLogins");
			if (parameters.get("rememberMe") != null
					&& !parameters.get("rememberMe").equals("false")
					&& allowCustomerAutoLogins != null
					&& allowCustomerAutoLogins.equals("1")) {
				setAutoLoginCookie = true;
				parameters.put("autoLoginToken", baseForm.getRequest()
						.getSession().getId());
			}

			// Rearrange the parameters if this is coming from the LoginAndUpdatePassword action
			if (parameters.get("currentPassword") != null) {
				parameters.put("newDecryptedPassword", parameters.get("decryptedPassword"));
				parameters.put("decryptedPassword", parameters.get("currentPassword"));
			}

			// Process the login
			CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerProcessorImplementer");
			Map result = customerProcessor.processLogin(parameters);

			// Process errors
			if (result != null && result.get("resultCode") != null
					&& !result.get("resultCode").equals("0")) {
				String resultCode = (String) result.get("resultCode");

				// Redirect to a special password update form without logging the customer in
				if (resultCode.equals("errors.passwordHasExpired")) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(resultCode));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
					return mapping.findForward("passwordHasExpired");
				} else if (resultCode.equals("errors.sameAsPreviousPassword")) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(resultCode,
							baseForm.getSettings().getValue("numberOfPreviousPasswordsToCheckCustomer")));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
					if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
						log.info("Validation error(s): " + baseForm.getErrors().toString());
					return mapping.findForward("failure");

				// Act successfully if user is already logged in
				} else if (!resultCode.equals("errors.userAlreadyLoggedIn")) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage(resultCode));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
							.getErrors());
					if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
						log.info("Validation error(s): "
								+ baseForm.getErrors().toString());
					return mapping.findForward("failure");
				}
			}

			// Set the autoLoginToken cookie
			if (setAutoLoginCookie) {
				String autoLoginDays = (String) baseForm.getSettings()
						.getValue("autoLoginDays");
				int maxAge = 86400000; // Default to 10000 days
				if (autoLoginDays != null && !autoLoginDays.equals("0")) {
					try {
						maxAge = Integer.parseInt(autoLoginDays) * 86400;
					} catch (NumberFormatException nfe) {
						if (log.isInfoEnabled())
							log.info(nfe);
					}
				}
				Cookie autoLoginCookie = new Cookie("sscaltoken", baseForm
						.getUser().getCustomer().getAutoLoginToken());
				autoLoginCookie.setMaxAge(maxAge);
				response.addCookie(autoLoginCookie);
			}

			// Force shipping and taxes to be recalculated
			baseForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "false");

			// Forward to the original request if any
			String originalRequest = (String) parameters.get("originalRequest");
			if (originalRequest != null && originalRequest.length() > 0) {
				originalRequest = originalRequest.replaceFirst(request
						.getContextPath(), "");
				if (log.isInfoEnabled())
					log.info("Forwarding request after login to: "
							+ originalRequest);
				RequestDispatcher rd = request
						.getRequestDispatcher(originalRequest);
				rd.forward(request, response);
				return null;
			}

			return mapping.findForward("success");
		}
	}
}
