package com.softslate.commerce.customer.payment;

import java.util.Map;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.payment.IPayPalExpressPaymentProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Action class to handle a request from the user to use PayPal Express to
 * checkout. Corresponds to the url "/PayPalExpressSet.do".
 * 
 * As required by PayPal, users can choose to use PayPal Express in two
 * locations: on the cart screen, and on the checkout payment screen. In either
 * case, this action is executed, which triggers the first of the three API
 * calls the PayPal Express: SetExpressCheckout.
 * 
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.payment.PayPalExpressProcessor#processSetExpressCheckout(Map)},
 * or whatever class is defined in the "activePaymentProcessors" setting that
 * corresponds to the paypalExpressForm.jsp template, to communicate with
 * PayPal.
 * 
 * @author David Tobey
 */
public class PayPalExpressSetAction extends BaseAction {
	static Log log = LogFactory.getLog(PayPalExpressSetAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		// Initialize the BaseForm
		BaseForm baseForm = new BaseForm();
		baseForm.initializeProperties(mapping, request);

		synchronized (baseForm.getRequest().getSession()) {

			// Send to the cart page if the order is not established
			if (baseForm.getUser().getOrder() == null
					|| baseForm.getUser().getOrder().getSubtotal() == null) {
				return baseForm.getMapping().findForward("failure");
			}

			// Use the API to call SetExpressCheckout and get the user's token
			String processorName = lookUpPayPalProcessorName();
			IPayPalExpressPaymentProcessor ppep = (IPayPalExpressPaymentProcessor) baseForm
					.getBusinessObjectFactory().createObjectFromClassName(
							processorName);
			Map results = ppep.processSetExpressCheckout(null);
			String resultCode = (String) results.get("resultCode");
			String resultMessages[] = (String[]) results.get("resultMessages");

			// Handle the response
			if (null == resultCode)
				resultCode = new String("-1");

			if (!resultCode.equals("0")) {
				if (null != resultMessages && resultMessages.length > 0)
					for (int i = 0; i < resultMessages.length; i++)
						baseForm.getErrors().add(
								ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.detail",
										resultMessages[i]));
				else
					baseForm.getErrors().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail", resultCode
									+ " : " + "Unknown PayPal Error"));

				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				return baseForm.getMapping().findForward("failure");
			} else {
				// Now redirect to paypalExpressURL
				String paypalEnvironment = (String) baseForm.getSettings()
						.getValue("paypalEnvironment");
				String paypalToken = (String) baseForm.getUser().getCustomMap()
						.get("paypalToken");
				String paypalExpressURL = "https://www.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token="
						+ paypalToken;
				if (paypalEnvironment.equals("sandbox")) {
					if (processorName.indexOf("UK") > 0)
						paypalExpressURL = "https://test-expresscheckout.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token="
								+ paypalToken;
					else
						paypalExpressURL = "https://www.sandbox.paypal.com/cgi-bin/webscr?cmd=_express-checkout&token="
								+ paypalToken;
				}
				response.sendRedirect(paypalExpressURL);
			}

			return baseForm.getMapping().findForward("success");
		} // end of synchronized block
	}

	public String lookUpPayPalProcessorName() {
		// Look up the class name of the PayPal Processor, using the PayPal
		// template as a basis
		String activePaymentFormTemplates = ((Properties) getServlet()
				.getServletContext().getAttribute("appComponents"))
				.getProperty("activePaymentFormTemplates");
		String activePaymentProcessors = ((Properties) getServlet()
				.getServletContext().getAttribute("appComponents"))
				.getProperty("activePaymentProcessors");
		String[] templates = activePaymentFormTemplates.split(",");
		String[] processors = activePaymentProcessors.split(",");
		for (int i = 0; i < templates.length; i++) {
			if (templates[i].indexOf("paypalExpress") != -1 && processors.length > i) {
				return processors[i];
			}
		}
		return "com.softslate.commerce.businessobjects.payment.PayPalExpressProcessor";
	}
}
